@@ -32,6 +32,7 @@ from traydevice import __name__ as package_name
 from traydevice import __data_dir__ as package_data
 
 
+
 def get_resource(resource):
     """
         Retrieve resource files
@@ -40,7 +41,7 @@ def get_resource(resource):
     return os.path.join(_resource_dir, resource)
 
 
-def get_config_file(filename):
+def get_config_file(filename, get_name_only=False):
     """
         Retrieve path to configuration file identified by
         filename. If file does not exist at location defined by
@@ -48,6 +49,8 @@ def get_config_file(filename):
     """
     xdg = os.path.join(
         BaseDirectory.save_config_path('traydevice'), filename)
+    if get_name_only:
+        return xdg
     if not os.path.exists(xdg):
         shutil.copyfile(get_resource(filename), xdg)
     return xdg
@@ -60,39 +63,44 @@ class Main:
             Initialize traydevice, parse command line, read configuration
         """
         _prog = "%prog"
-        configfile = get_config_file('default.xml')
         parser = OptionParser(usage="%prog [options] <device_file>",
                               version="%s %s" % (_prog, package_version))
+        default_config_name='default.xml'
         parser.add_option('-c', '--configfile',
             dest='configfile',
             help='read configuration from FILE instead of default in %s' %
-                configfile, metavar='FILE')
+                get_config_file(default_config_name, True), metavar='FILE')
         (opts, args) = parser.parse_args()
-        if opts.configfile != None:
+        if not opts.configfile:
+            configfile = get_config_file(default_config_name)
+        else:
             configfile = opts.configfile
 
         if len(args) != 1:
-            logging.getLogger('Main').error('device_file argument is required')
+            logging.getLogger(__name__).error('device_file argument is required')
             sys.exit(1)
         try:
             configuration = self.__open_configuration(configfile)
         except Exception as e:
-            logging.getLogger('Main').error(
+            logging.getLogger(__name__).error(
                 'Cannot read configuration file \'%s\' (%s)'
                 % (configfile, e))
             sys.exit(1)
         try:
-            self.device = device.Device(args[0], self)
+            configured_backend = configuration.xpath('/traydevice/@backend')
+            if configured_backend:
+                self.device = device.create_device(args[0], self, configured_backend[0])
+            else:
+              self.device = device.create_device(args[0], self)
         except Exception as e:
-            logging.getLogger('Main').error(
-                'Cannot access device \'%s\' (%s)'
-                % (args[0], e))
+            logging.getLogger(__name__).exception('Cannot access device \'%s\''%args[0]);
             sys.exit(1)
         try:
             self.gui = gui.DeviceGui(configuration, self.device)
         except Exception as e:
-            logging.getLogger('Main').error(
-                'Gui construction failed. (%s)' % e)
+            logging.getLogger(__name__).exception('Gui construction failed.');
+#            logging.getLogger(__name__).error(
+#                'Gui construction failed. (%s)' % e)
             sys.exit(1)
 
     def start(self):
@@ -113,6 +121,7 @@ class Main:
         """
             Device.device_removed_listener callback
         """
+        logging.getLogger(__name__).debug('Terminating...');
         self.stop()
 
     def __open_configuration(self, configuration_file):
@@ -137,7 +146,11 @@ def main():
         Start traydevice
     """
     signal.signal(signal.SIGINT, signal.SIG_DFL)
-    logging.config.fileConfig(get_resource('logging.conf'))
+    try:
+      logging.config.fileConfig(get_resource('logging.conf'))
+    except:
+      logging.basicConfig(level=logging.DEBUG)
+      logging.getLogger('root').warn('Failed to configure logging from %s'%get_resource('logging.conf'), exc_info=1)
     main = Main()
     main.start()
 
